# Generated by roxygen2: do not edit by hand

S3method(adjust_pred_scale,AdaBoostGBMDist)
S3method(adjust_pred_scale,BernoulliGBMDist)
S3method(adjust_pred_scale,CoxPHGBMDist)
S3method(adjust_pred_scale,GammaGBMDist)
S3method(adjust_pred_scale,GaussianGBMDist)
S3method(adjust_pred_scale,HuberizedGBMDist)
S3method(adjust_pred_scale,LaplaceGBMDist)
S3method(adjust_pred_scale,PairwiseGBMDist)
S3method(adjust_pred_scale,PoissonGBMDist)
S3method(adjust_pred_scale,QuantileGBMDist)
S3method(adjust_pred_scale,TDistGBMDist)
S3method(adjust_pred_scale,TweedieGBMDist)
S3method(as.double,GBMTPerformance)
S3method(check_dist_params,AdaBoostGBMDist)
S3method(check_dist_params,BernoulliGBMDist)
S3method(check_dist_params,CoxPHGBMDist)
S3method(check_dist_params,GammaGBMDist)
S3method(check_dist_params,GaussianGBMDist)
S3method(check_dist_params,HuberizedGBMDist)
S3method(check_dist_params,LaplaceGBMDist)
S3method(check_dist_params,PairwiseGBMDist)
S3method(check_dist_params,PoissonGBMDist)
S3method(check_dist_params,QuantileGBMDist)
S3method(check_dist_params,TDistGBMDist)
S3method(check_dist_params,TweedieGBMDist)
S3method(check_response_values,AdaBoostGBMDist)
S3method(check_response_values,BernoulliGBMDist)
S3method(check_response_values,CoxPHGBMDist)
S3method(check_response_values,GammaGBMDist)
S3method(check_response_values,GaussianGBMDist)
S3method(check_response_values,HuberizedGBMDist)
S3method(check_response_values,LaplaceGBMDist)
S3method(check_response_values,PairwiseGBMDist)
S3method(check_response_values,PoissonGBMDist)
S3method(check_response_values,QuantileGBMDist)
S3method(check_response_values,TDistGBMDist)
S3method(check_response_values,TweedieGBMDist)
S3method(create_cv_groups,BernoulliGBMDist)
S3method(create_cv_groups,PairwiseGBMDist)
S3method(create_cv_groups,default)
S3method(create_dist,AdaBoostGBMDist)
S3method(create_dist,BernoulliGBMDist)
S3method(create_dist,CoxPHGBMDist)
S3method(create_dist,GammaGBMDist)
S3method(create_dist,GaussianGBMDist)
S3method(create_dist,HuberizedGBMDist)
S3method(create_dist,LaplaceGBMDist)
S3method(create_dist,PairwiseGBMDist)
S3method(create_dist,PoissonGBMDist)
S3method(create_dist,QuantileGBMDist)
S3method(create_dist,TDistGBMDist)
S3method(create_dist,TweedieGBMDist)
S3method(create_dist,default)
S3method(determine_groups,PairwiseGBMDist)
S3method(determine_groups,default)
S3method(distribution_name,GBMDist)
S3method(distribution_name,GBMFit)
S3method(distribution_name,default)
S3method(gbm_call_dist_name,PairwiseGBMDist)
S3method(gbm_call_dist_name,default)
S3method(gbm_cv_errors,GBMCVFit)
S3method(get_misc,AdaBoostGBMDist)
S3method(get_misc,BernoulliGBMDist)
S3method(get_misc,CoxPHGBMDist)
S3method(get_misc,GammaGBMDist)
S3method(get_misc,GaussianGBMDist)
S3method(get_misc,HuberizedGBMDist)
S3method(get_misc,LaplaceGBMDist)
S3method(get_misc,PairwiseGBMDist)
S3method(get_misc,PoissonGBMDist)
S3method(get_misc,QuantileGBMDist)
S3method(get_misc,TDistGBMDist)
S3method(get_misc,TweedieGBMDist)
S3method(get_misc,default)
S3method(get_ylabel,AdaBoostGBMDist)
S3method(get_ylabel,BernoulliGBMDist)
S3method(get_ylabel,CoxPHGBMDist)
S3method(get_ylabel,GammaGBMDist)
S3method(get_ylabel,GaussianGBMDist)
S3method(get_ylabel,HuberizedGBMDist)
S3method(get_ylabel,LaplaceGBMDist)
S3method(get_ylabel,PairwiseGBMDist)
S3method(get_ylabel,PoissonGBMDist)
S3method(get_ylabel,QuantileGBMDist)
S3method(get_ylabel,TDistGBMDist)
S3method(get_ylabel,TweedieGBMDist)
S3method(get_ylabel,default)
S3method(get_ylabel_one_var,BernoulliGBMDist)
S3method(get_ylabel_one_var,PairwiseGBMDist)
S3method(get_ylabel_one_var,PoissonGBMDist)
S3method(get_ylabel_one_var,default)
S3method(interact,GBMFit)
S3method(loss,AdaBoostGBMDist)
S3method(loss,BernoulliGBMDist)
S3method(loss,CoxPHGBMDist)
S3method(loss,GammaGBMDist)
S3method(loss,GaussianGBMDist)
S3method(loss,HuberizedGBMDist)
S3method(loss,LaplaceGBMDist)
S3method(loss,PairwiseGBMDist)
S3method(loss,PoissonGBMDist)
S3method(loss,QuantileGBMDist)
S3method(loss,TDistGBMDist)
S3method(loss,TweedieGBMDist)
S3method(loss,default)
S3method(plot,GBMFit)
S3method(plot,GBMTPerformance)
S3method(predict,GBMCVFit)
S3method(predict,GBMFit)
S3method(print,GBMFit)
S3method(print,GBMTPerformance)
S3method(print,gbmParallel)
S3method(reorder_fit,CoxPHGBMDist)
S3method(reorder_fit,PairwiseGBMDist)
S3method(reorder_fit,default)
S3method(response,BernoulliGBMDist)
S3method(response,GammaGBMDist)
S3method(response,PairwiseGBMDist)
S3method(response,PoissonGBMDist)
S3method(response,TweedieGBMDist)
S3method(response,default)
S3method(summary,GBMFit)
S3method(update_fold_dist_data,CoxPHGBMDist)
S3method(update_fold_dist_data,PairwiseGBMDist)
S3method(update_fold_dist_data,default)
S3method(update_num_train_groups,PairwiseGBMDist)
S3method(update_num_train_groups,default)
S3method(weight_group_consistency,GBMDist)
S3method(weight_group_consistency,PairwiseGBMDist)
S3method(weight_group_consistency,default)
export(available_distributions)
export(baseline_hazard)
export(calibrate_plot)
export(distribution_name)
export(gbm)
export(gbm.fit)
export(gbm.perf)
export(gbmParallel)
export(gbm_dist)
export(gbm_more)
export(gbm_roc_area)
export(gbmt)
export(gbmt_fit)
export(gbmt_performance)
export(interact)
export(iteration_error)
export(perf_pairwise)
export(permutation_relative_influence)
export(pretty_gbm_tree)
export(quantile_rug)
export(relative_influence)
export(to_old_gbm)
export(training_params)
export(trees)
import(lattice)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,rug)
importFrom(splines,ns)
importFrom(stats,approx)
importFrom(stats,binomial)
importFrom(stats,delete.response)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,loess)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.pass)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,runif)
importFrom(stats,supsmu)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
useDynLib(gbm3)
